package br.com.jbpm.manager;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceUnit;

import org.apache.log4j.Logger;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.SystemEventListenerFactory;
import org.drools.container.spring.beans.persistence.DroolsSpringJpaManager;
import org.drools.persistence.PersistenceContextManager;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.process.ProcessInstance;
import org.jbpm.process.workitem.wsht.AsyncHornetQHTWorkItemHandler;
import org.jbpm.task.Group;
import org.jbpm.task.User;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.TaskService;
import org.jbpm.task.service.TaskServiceSession;
import org.jbpm.task.service.hornetq.HornetQTaskClientConnector;
import org.jbpm.task.service.hornetq.HornetQTaskClientHandler;
import org.jbpm.task.service.hornetq.HornetQTaskServer;

import br.com.impl.knowledge.EasyPieKnowledgeBaseImpl;
import br.com.impl.knowledge.EasyPieKnowledgeSessionImpl;

public class ProcessManager implements Serializable {

	@PersistenceUnit(unitName = "org.jbpm.process")
	private EntityManagerFactory				entityManagerFactoryJbpm;

	private static final long					serialVersionUID = 1L;

	private static final Logger					LOGGER			 = Logger.getLogger(ProcessManager.class);

	private static EasyPieKnowledgeBaseImpl		kBaseImpl		 = new EasyPieKnowledgeBaseImpl();
	
	private static KnowledgeBase 				knowledgeBase;
	private Environment 						environment;
	
	/**
	 * Nesse método o processo é iniciado e persistido no banco. <br/>
	 * A sequência de passos é a seguinte: <br/>
	 * 1 - Usuários falsos são adicionados na sessão. <br/>
	 * 2 - É criado uma nova sessão persistente onde o processo irá ser executado. <br/>
	 * 3 - Handlers para lidar com as tasks são registrados. <br/>
	 * 4 - Um map de parâmetros é criado contendo todos os parâmetros iniciais do processo. <br/>
	 * 5 - É criado uma instância do processo e a mesma é inserida na sessão. <br/>
	 * 6 - A instância do processo é iniciada e seu id retornado <br/>
	 * @return O id do processo iniciado.
	 */
	public Long startProcess(String usuarioLogado) {

		TaskService taskService = null;
		
		getEntityManagerFactoryJbpm();
		
		taskService = new TaskService(entityManagerFactoryJbpm, SystemEventListenerFactory.getSystemEventListener());
		
		try{
			HornetQTaskServer server = new HornetQTaskServer(taskService, "localhost", 5445);
			if(!server.isRunning()) {
				Thread thread = new Thread(server);
				thread.start();
			}
		} catch(Exception e) {
			System.out.println("HornetQ Task Server já iniciado");
		}
		
		TaskServiceSession taskSession = taskService.createSession();
		
		createLdapCallback(taskSession);

		SessionManager sessionManager = new SessionManager();

		sessionManager.createNewPersistentSession();

		registerHandlers(sessionManager.getKnowledgeSession());

		final Map<String, Object> params = new HashMap<String, Object>();
		params.put("user", usuarioLogado);

		ProcessInstance processInstance = sessionManager.getSession().startProcess("defaultPackage.DemoProcess", params);
		
		return processInstance.getId();

	}

	/**
	 * Registra na sessão os handlers necessários para lidar com as tasks
	 */
	public static void registerHandlers(EasyPieKnowledgeSessionImpl kSession) {

		LOGGER.info("REGISTERING HANDLERS...");

		TaskClient client = new TaskClient(new HornetQTaskClientConnector(UUID.randomUUID().toString(), new HornetQTaskClientHandler(SystemEventListenerFactory.getSystemEventListener())));
		//		client.connect("localhost", 5445);
		TaskManager taskManager = new TaskManager();

		AsyncHornetQHTWorkItemHandler htHandler = taskManager.getHumanTaskService().createTaskHandler(kSession.getKsession(), client);
		htHandler.setPort(5445);
		htHandler.connect();
		kSession.registerWorkItemHandler("Human Task", htHandler);

	}

	public static void createLdapCallback(TaskServiceSession taskSession) {

		LOGGER.info("CREATE LDAP CALLBACK");
		
		final User guilhermetelles = new User("guilherme.telles");
		final User leonardosiqueira = new User("leonardo.siqueira");
		final User administrator = new User("Administrator");

		final Group requisitante = new Group("requisitante");

		taskSession.addUser(administrator);
		taskSession.addUser(guilhermetelles);
		taskSession.addUser(leonardosiqueira);
		taskSession.addGroup(requisitante);

	}

	public static KnowledgeBase getKnowledgeBase() {

		if(ProcessManager.knowledgeBase == null) {
			LOGGER.info("LOADING KNOWLEDGE BASE");

			knowledgeBase = kBaseImpl.readKnowledgeBase();
		}

		return knowledgeBase;
	}

	public Environment getEnvironment() {

		LOGGER.info("CREATING NEW ENVIRONMENT");

		environment = KnowledgeBaseFactory.newEnvironment();
		environment.set(EnvironmentName.ENTITY_MANAGER_FACTORY, getEntityManagerFactoryJbpm());

		PersistenceContextManager persistenceContextManager = new DroolsSpringJpaManager(environment);
		environment.set(EnvironmentName.PERSISTENCE_CONTEXT_MANAGER, persistenceContextManager);

		return environment;
	}

	public EntityManagerFactory getEntityManagerFactoryJbpm() {

		LOGGER.info("CREATING NEW ENTITY MANAGER FACTORY");

		entityManagerFactoryJbpm = Persistence.createEntityManagerFactory("org.jbpm.process");

		return entityManagerFactoryJbpm;
	}

}
