package br.com.jbpm.manager;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.drools.SystemEventListenerFactory;
import org.jbpm.task.Content;
import org.jbpm.task.Status;
import org.jbpm.task.Task;
import org.jbpm.task.TaskData;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.TaskClient;
import org.jbpm.task.service.TaskService;
import org.jbpm.task.service.hornetq.HornetQTaskClientConnector;
import org.jbpm.task.service.hornetq.HornetQTaskClientHandler;
import org.jbpm.task.utils.ContentMarshallerHelper;
import org.springframework.stereotype.Component;

import br.com.impl.task.EasyPieLocalTaskServiceImpl;

@Component
public class TaskManager {

	private static final Logger	LOGGER = Logger.getLogger(TaskManager.class);

	private EasyPieLocalTaskServiceImpl	humanTaskService;
	private TaskClient					taskClient;

	/**
	 * Define a task especificada para o ator passado.
	 * @param taskId - Id da task
	 * @param actor - Nome do ator da task
	 * @return O id da task iniciada
	 */
	public long claimTask(final Long taskId, final String actor) {

		LOGGER.info("Claim task");

		getHumanTaskService().claim(taskId, actor);

		LOGGER.info("Claim task - Finish");

		return taskId;
	}

	/**
	 * Define a próxima task disponível do processo para o ator passado.
	 * @param actor - Nome do ator da task.
	 * @param processId - Id do processo
	 * @return O id da task iniciada
	 */
	public long claimTask(final String actor,  final Long processId) {

		LOGGER.info("Claim task");

		final List<Status> status = new ArrayList<Status>();
		status.add(Status.Ready);

		long id = findNextTaskIdByProcess(processId, actor, status);

		if(id != 0l) {
			getHumanTaskService().claim(id, actor);
		}

		LOGGER.info("Claim task - Finish");

		return id;
	}

	/**
	 * Inicia a task especificada com o ator passado.
	 * @param taskId - Id da task
	 * @param actor - Nome do ator da task
	 * @return O id da task iniciada
	 */
	public long startTask(final Long taskId, final String actor) {

		LOGGER.info("Start task");

		getHumanTaskService().start(taskId, actor);

		LOGGER.info("Start task - Finish");

		return taskId;
	}

	/**
	 * Inicia a próxima task do processo e do ator especificado.
	 * @param actor - Nome do ator da task.
	 * @param processId - Id do processo
	 * @return O id da task iniciada
	 */
	public long startTask(final String actor, final Long processId) {

		LOGGER.info("Start task");

		final List<Status> status = new ArrayList<Status>();
		status.add(Status.Reserved);

		long id = findNextTaskIdByProcess(processId, actor, status);

		if(id != 0l) {

			LOGGER.info("Start task - Middle");

			getHumanTaskService().start(id, actor);
		}

		LOGGER.info("Start task - Finish");

		return id;
	}

	/**
	 * Completa a task especificada com o ator e resultado passados.
	 * @param taskId - Id da task
	 * @param actor - Nome do ator da task
	 * @param results - Map de resultados da task.
	 */
	public void completeWithResults(final Long taskId, final String actor, final Map<String, Object> results) {

		LOGGER.info("Complete task " + taskId);

		getHumanTaskService().completeWithResults(taskId, actor, results);

		LOGGER.info("Complete task - Finish");
	}

	/**
	 * Completa a próxima task disponível do processo e ator com resultado especificados.
	 * @param actor - Nome do ator da task
	 * @param results - Map de resultados da task.
	 * @param processId - Id do processo
	 */
	public void completeWithResults(final String actor,  final Long processId, final Map<String, Object> results) {

		LOGGER.info("Complete task");

		final List<Status> status = new ArrayList<Status>();
		status.add(Status.InProgress);
		long taskId = findNextTaskIdByProcess(processId, actor, status);

		getHumanTaskService().completeWithResults(taskId, actor, results);

		LOGGER.info("Complete task - Finish");
	}

	/**
	 * Pega a variável especificada de uma task.
	 * @param taskId - Id da task
	 * @return a variável desejada
	 */
	public Object getTaskVariable(final Long taskId, final String variable) {

		final Map<?, ?> taskVariables = getTaskVariables(taskId);
		return taskVariables.get(variable);
	}

	/**
	 * Puxa um map de todas variáveis da task especificada.
	 * 
	 * @param taskId - Id da task
	 * @return Map de todas as variáveis da tarefa
	 */
	public Map<?, ?> getTaskVariables(final Long taskId) {

		final Task task = getHumanTaskService().getTask(taskId);
		final TaskData taskData = task.getTaskData();

		final Long documentContentId = taskData.getDocumentContentId();

		final Content content = getHumanTaskService().getContent(documentContentId);

		return ((Map<?, ?>) ContentMarshallerHelper.unmarshall(content.getContent(), null));
	}

	private Long findNextTaskIdByProcess(Long processId, String actor, List<Status> status){

		final List<TaskSummary> tasks = getHumanTaskService().getTasksAssignedAsPotentialOwnerByStatus(actor, status, "en-UK");

		if(!tasks.isEmpty()) {

			LOGGER.info("Tasks Not Empty");

			int taskSize = tasks.size();
			for(int i = taskSize; i > 0; i--){
				TaskSummary taskSummary = tasks.get(i-1);
				Task task = getHumanTaskService().getTask(taskSummary.getId());
				Long taskProcessId = task.getTaskData().getProcessInstanceId();
				long id = task.getId();

				if(taskProcessId.equals(processId)) {
					return id;
				}
			}
		}

		long id = findNextTaskIdByProcess(processId, actor, status);

		return id;
	}


	public EasyPieLocalTaskServiceImpl getHumanTaskService() {

		if (humanTaskService == null) {

			LOGGER.info("Generating Human Task Service");

			TaskService taskService = new TaskService(new ProcessManager().getEntityManagerFactoryJbpm(), SystemEventListenerFactory.getSystemEventListener());

			ProcessManager.createLdapCallback(taskService.createSession());

			humanTaskService = new EasyPieLocalTaskServiceImpl();

			humanTaskService.assignTaskClient(getTaskClient());

		}

		return humanTaskService;
	}

	public void setHumanTaskService(final EasyPieLocalTaskServiceImpl humanTaskService) {
		this.humanTaskService = humanTaskService;
	}

	public TaskClient getTaskClient() {

		if(taskClient == null) {
			taskClient = new TaskClient(new HornetQTaskClientConnector("client" + UUID.randomUUID(), new HornetQTaskClientHandler(SystemEventListenerFactory.getSystemEventListener())));
			taskClient.connect("localhost", 5445);
		}

		return taskClient;
	}

}
